@@ -3,8 +3,8 @@
  *
  *   (C) 2003 Dominic Morris
  *
- *   $Id: dongle.c,v 1.1 2003/11/18 22:44:52 dom Exp $
- *   $Date: 2003/11/18 22:44:52 $
+ *   $Id: dongle.c,v 1.3 2003/11/19 23:48:13 dom Exp $
+ *   $Date: 2003/11/19 23:48:13 $
  *
  *
  *   Dongle registration - maintains a list of dongles with their subcomponents 
@@ -26,20 +26,20 @@ dongle_t *dongle_return(char *address)
     int        first = -1;
 
     for ( i = 0; i < dongles_num; i++ ) {
-	if ( strcmp(dongles[i].address,address) == 0 && dongles[i].inuse ) {
-	    return &dongles[i];
-	}
-	if ( dongles[i].inuse == FALSE ) {
-	    first = i;
-	}
+        if ( strcmp(dongles[i].address,address) == 0 && dongles[i].inuse ) {
+            return &dongles[i];
+        }
+        if ( dongles[i].inuse == FALSE ) {
+            first = i;
+        }
     }
 
     if ( first != -1 ) {
-	dongle=&dongles[first];
+        dongle=&dongles[first];
     } else {
-	i = dongles_num++;
-	dongles = realloc(dongles,dongles_num * sizeof(dongle_t));
-	dongle = &dongles[i];
+        i = dongles_num++;
+        dongles = realloc(dongles,dongles_num * sizeof(dongle_t));
+        dongle = &dongles[i];
     }
 
 
@@ -48,6 +48,12 @@ dongle_t *dongle_return(char *address)
     dongle->gui = NULL;
     dongle->stream = NULL;
 
+    /* Reset to defaults - we ask client for details of these settings */
+    dongle->tvmode = 0;
+    dongle->videooutput = 0;
+    dongle->flickermode = 0;
+    dongle->aspectratio = 0;
+
     return dongle;
 }
 
@@ -61,3 +67,14 @@ void dongle_close(dongle_t *dongle)
     /* Close gui/stream here */
     dongle->inuse = FALSE;
 }
+
+
+
+
+/*
+ * Local Variables:
+ *  indent-tabs-mode:nil
+ *  require-final-newline:t
+ *  c-basic-offset: 4
+ * End:
+ */
