@@ -3,8 +3,8 @@
  *
  *   (C) 2003 Dominic Morris
  *
- *   $Id: gui.c,v 1.17 2003/11/19 23:48:13 dom Exp $
- *   $Date: 2003/11/19 23:48:13 $
+ *   $Id: gui.c,v 1.22 2003/11/24 20:34:37 dom Exp $
+ *   $Date: 2003/11/24 20:34:37 $
  *
  *
  *   Handles the GUI...
@@ -16,6 +16,11 @@
 #define NEED_MSGS
 #include "msgs.h"
 
+typedef struct {
+    void    *param;
+    int    (*ack_func)(int acktype, void *param, uint8_t *buf, int len);
+} ackhandle_t;
+
 typedef struct {
     void    *param;
     int    (*key_func)(void *menu,int keycode); /* Key press callback */
@@ -28,9 +33,10 @@ struct _gui {
     int           state;
     uint8_t       send_update;      /* Send an update out when necessary */
     render_t     *render;           /* Rendering surface */
-    int           keys_active;      /* Which set of keys is active */
     keyhandle_t   menus;
     keyhandle_t   media;
+    ackhandle_t   menus_ack;
+    ackhandle_t   media_ack;
     app_t        *app;              /* Controlling application */
     char          inbuf[1024];
     int           inbufstart;
@@ -102,6 +108,7 @@ gui_t *new_gui(dongle_t *dongle)
     gui->app    = new_app(dongle,gui->render,gui);
     gui->inbufstart = 0;
 
+
     return gui;
 }
 
@@ -314,17 +321,16 @@ static int rfb_running(gui_t *gui, unsigned char *buf, int len)
                     dongle_close(gui->dongle);
                     return -1;
                 }
-
-                if ( gui->keys_active == KEYS_MENU && gui->menus.key_func ) {
-                    (gui->menus.key_func)(gui->menus.param,code);
-                } else if ( gui->keys_active == KEYS_MEDIA && gui->media.key_func ) {
+		if (  gui->media.key_func ) {
                     /* Now pass the key onto the menu stuff if necessary */
                     if ( (gui->media.key_func)(gui->media.param,code) == 0 ) {
                         if ( gui->menus.key_func ) {
                             (gui->menus.key_func)(gui->menus.param,code);
                         }
                     }
-                }
+                } else if ( gui->menus.key_func ) {
+		    (gui->menus.key_func)(gui->menus.param,code);
+		}
             }	    
         
             diff = 8;
@@ -350,57 +356,47 @@ static int rfb_running(gui_t *gui, unsigned char *buf, int len)
             }
         }
         break;
-    case RFB_MEDIA_ACK:     
+    case RFB_RDC_ACK:     
         if ( len >= 10 ) {
             diff = 10;
-	    dump_hex(buf,len);	    
-            switch ( buf[1] ) {		
-            case RFB_MEDIA_FILENAME:
-                if ( (stream_get_type(gui->dongle->stream) & MEDIA_MASK) == MEDIA_MPEG ) {
-                    gui_send_message(gui,&rfb_media_mpeg);
-                    gui_send_message(gui,&rfb_display_off);
-                }
-                break;
-            case RFB_MEDIA_STOP:
-                if ( ( stream_get_type(gui->dongle->stream) & MEDIA_MASK ) == MEDIA_MPEG ) {
-                    gui_send_message(gui,&rfb_media_mpeg);
-                    gui_send_message(gui,&rfb_display_on);
-                } else {
-		    /* It's audio, so we could well stream something else,
-		       use the callback function 
-		    */
-#if 0
-		    if ( gui->next_media ) {
-			(gui->next_media)(void *param);
-		    }
-#endif
-		}
-                break;
-            case RFB_MEDIA_VOLDOWN:
-            case RFB_MEDIA_VOLUP:
-            case RFB_MEDIA_MUTE:
-                break;
-            case RFB_MEDIA_MPEG:
-                printf("Screen state switch received\n");
-                break;
-	    case RFB_MEDIA_SETTINGS:
-		if ( buf[7] != 0x00 ) {
-		    if ( diff + buf[7] >= len ) {		   
-			diff += buf[7];
+
+            switch ( buf[1] ) {		          
+            case RDC_SETTINGS:       /* This really is the only one we need to catch */
+                if ( buf[7] != 0x00 ) {
+                    if ( diff + buf[7] >= len ) {		   
+                        diff += buf[7];
 		    
-			/* Save those settings */
-			gui->dongle->tvmode = buf[11];
-			gui->dongle->videooutput = buf[12];
-			gui->dongle->flickermode = buf[13];
-			gui->dongle->aspectratio = buf[14];
-			printf("TVmode %d output = %d flicker = %d aspect = %d\n",buf[11],buf[12],buf[13],buf[14]);
-		    } else {
-			diff = 0;
-		    }		   
-		}
-		break;
+                        /* Save those settings */
+                        gui->dongle->tvmode = buf[11];
+                        /* output mode seems to be supplied with -1 - quite why I've no idea! */
+                        gui->dongle->videooutput = buf[12] - 1;
+                        gui->dongle->flickermode = buf[13];
+                        gui->dongle->aspectratio = buf[14];
+                        printf("TVmode %d output = %d flicker = %d aspect = %d\n",
+                               gui->dongle->tvmode,
+                               gui->dongle->videooutput,
+                               gui->dongle->flickermode,
+                               gui->dongle->aspectratio);
+
+                    } else {
+                        diff = 0;  /* Not had all the data yet */
+                    }		   
+                } 
+                break;
             default:
-                printf("Can't deal with MEDIA_ACK type %d\n",buf[1]);
+                /* Kick ack events up to a higher level so we can do multiple plays */
+		if ( gui->media_ack.ack_func ) {
+                    /* Now pass the key onto the menu stuff if necessary */
+                    if ( (gui->media_ack.ack_func)(buf[1],gui->media_ack.param,buf,len) == 0 ) {
+                        if ( gui->menus_ack.ack_func ) {
+                            (gui->menus_ack.ack_func)(buf[1],gui->menus_ack.param,buf,len);
+                        }
+                    }
+                } else if ( gui->menus_ack.ack_func ) {
+		    (gui->menus_ack.ack_func)(buf[1],gui->menus_ack.param,buf,len);
+		} else {
+                    printf("Can't deal with MEDIA_ACK type %d\n",buf[1]);
+                }
                 break;
             }
         }
@@ -461,7 +457,7 @@ static void rfb_send_data(gui_t *gui)
         strcpy(ptr,"rfbwindows");
         ptr += len;
         write(gui->fd,buf,ptr - buf);
-	gui_send_settings(gui,0);
+        gui_send_settings(gui,RDC_SETTINGS_GET);
         gui->state = VNC_RUNNING;
         break;    
     case VNC_RUNNING:        
@@ -569,7 +565,7 @@ static int deflateit(Byte *uncompr, uLong uncomprLen, Byte *compr, uLong comprLe
 /** \brief Register key handling functions
  *
  *  \param gui Handle the the gui channel
- *  \param types KEYS_MENU or KEYS_MEDIA
+ *  \param types REGISTER_MENU or REGISTER_MEDIA
  *  \param key_func Function to call back with key code
  *  \param param Generic pointer to pass back to the callback function
  *
@@ -579,17 +575,37 @@ static int deflateit(Byte *uncompr, uLong uncomprLen, Byte *compr, uLong comprLe
 void gui_register_keys(gui_t *gui,int type, 
 		       int (*key_func)(void *param,int keycode), void *param)
 {
-    if ( type == KEYS_MENU ) {
+    if ( type == REGISTER_MENU ) {
         gui->menus.key_func = key_func;
         gui->menus.param = param;
-        gui->keys_active = KEYS_MENU;
     } else {
         gui->media.key_func = key_func;
         gui->media.param = param;
-        gui->keys_active = KEYS_MEDIA;
     }
 }
 
+/** \brief Register ack handling functions
+ *
+ *  \param gui Handle the the gui channel
+ *  \param types REGISTER_MENU or REGISTER_MEDIA
+ *  \param key_func Function to call back with the ack
+ *  \param param Generic pointer to pass back to the callback function
+ *
+ *  We have two sets of functions for handling acks - one whilst in the
+ *  menu, and a second for playing media
+ */
+void gui_register_ack(gui_t *gui, int type,
+		      int    (*ack_func)(int acktype, void *param, unsigned char *buf, int len),
+		      void  *param)
+{
+    if ( type == REGISTER_MENU ) {
+        gui->menus_ack.ack_func = ack_func;
+        gui->menus_ack.param = param;
+    } else {
+        gui->media_ack.ack_func = ack_func;
+        gui->media_ack.param = param;
+    }
+}
 
 
 /** \brief Send a filename to the client down the RFB channel
@@ -601,16 +617,12 @@ void gui_register_keys(gui_t *gui,int type,
  * it before the client will accept it.
  *
  */  
-void gui_send_filename(gui_t *gui, char *name)
+void gui_send_play(gui_t *gui, char *name)
 {
     unsigned  char  buf[FILENAME_MAX + 24];
-    mvprfbmsg_t    *msg = &rfb_media_filename;
+    mvprfbmsg_t    *msg = &rfb_media_play;
     int             len = strlen(name);
 
-    /* If something is playing, then stop it! */
-    if ( stream_get_type(gui->dongle->stream) ) {
-        gui_send_message(gui,&rfb_media_stop);
-    }
 
     memcpy(buf,msg->msg,msg->len);
 
@@ -632,21 +644,7 @@ void gui_send_message(gui_t *gui, mvprfbmsg_t *msg)
     write(gui->fd,msg->msg,msg->len);
 }
 
-/** \brief Client has sent us an unexpected stop
- * 
- * \param gui Handle to the gui channel
- *
- * The client has send us a stop message, we should send a switch on screen
- * message and that should be enough to get us working again
- */
-void gui_notify_stop(gui_t *gui)
-{
-#if 0
-    if ( ( stream_get_type(gui->dongle->stream) & MEDIA_MASK) == MEDIA_MPEG ) {
-        gui_send_message(gui,&rfb_display_on);
-    }
-#endif
-}
+
 
 /** \brief Send the settings to the client down the RFB channel
  *
@@ -658,6 +656,7 @@ void gui_send_settings(gui_t *gui, int subcommand)
 
     gui_send_message(gui,&rfb_media_settings);
 
+   
     buf[0] = subcommand;    /* 0,1 (or 03??) */
     buf[1] = gui->dongle->tvmode;
     buf[2] = gui->dongle->videooutput;
@@ -666,7 +665,6 @@ void gui_send_settings(gui_t *gui, int subcommand)
     buf[5] = buf[6] = buf[7] = buf[8] = buf[9] = buf[10] = buf[11] = 0;
 
     write(gui->fd,buf,12);
-    dump_hex(buf,12);
 }
 
 /*
