@@ -2,8 +2,8 @@
  *  Header for MediaMVP server
  *
  *
- *  $Id: mediamvp.h,v 1.16 2003/11/19 23:48:13 dom Exp $
- *  $Date: 2003/11/19 23:48:13 $
+ *  $Id: mediamvp.h,v 1.21 2003/11/24 20:18:40 dom Exp $
+ *  $Date: 2003/11/24 20:18:40 $
  */
 
 #ifndef _MEDIAMVP_H_
@@ -18,6 +18,7 @@
 #include <string.h>
 #include <unistd.h>
 #include <errno.h>
+#include <stdint.h>
 #include <sys/socket.h>
 #include <arpa/inet.h>
 #include <netinet/in.h>
@@ -96,10 +97,13 @@ extern int             c_gui_port;
 extern void            gui_config();
 extern void            gui_init();
 extern void            delete_gui(gui_t *gui);
-#define  KEYS_MENU  1
-#define  KEYS_MEDIA 2
+#define  REGISTER_MENU  1
+#define  REGISTER_MEDIA 2
 extern void            gui_register_keys(gui_t *gui,int type,int (*key_func)(void *app,int keycode), void *menu);
-extern void            gui_send_filename(gui_t *gui, char *name);
+extern void            gui_register_ack(gui_t *gui, int type,
+                                        int    (*ack_func)(int acktype, void *param, unsigned char *buf, int len),
+                                        void  *param);
+extern void            gui_send_play(gui_t *gui, char *name);
 extern void            gui_send_message(gui_t *gui, mvprfbmsg_t *msg);
 extern void            gui_send_settings(gui_t *gui, int subcommand);
 
@@ -112,6 +116,11 @@ extern menu_t         *new_menu(render_t *render, menu_t *parent,app_t *app, gui
 extern void            delete_menu_tree(menu_t *menu);
 extern void            delete_menu(menu_t *menu);
 extern void            menu_set_title(menu_t *menu, char *text);
+#define   MENU_NONE    0
+#define   MENU_INT     1
+extern void            menu_add_option(menu_t *menu, char *name, 
+                                       int (callback_func)(menu_t *menu,app_t *app, int sel),
+                                       int type, int num_options, char **options, void *value_ptr);
 extern void            menu_add(menu_t *menu, char *name, int (*callback_func)(menu_t *menu,app_t *app, int sel));
 extern void            menu_display(menu_t *menu);
 
@@ -156,6 +165,7 @@ extern void            stream_config();
 extern void            stream_init();
 extern void            delete_stream(stream_t *stream);
 extern int             stream_get_type(stream_t *stream);
+extern int             stream_get_lasttype(stream_t *stream);
 
 
 /* udp.c */
