@@ -3,8 +3,8 @@
  *
  *   (C) 2003 Dominic Morris
  *
- *   $Id: menu.c,v 1.8 2003/11/19 23:48:13 dom Exp $
- *   $Date: 2003/11/19 23:48:13 $
+ *   $Id: menu.c,v 1.13 2003/11/24 20:18:40 dom Exp $
+ *   $Date: 2003/11/24 20:18:40 $
  *
  *
  *   Constructs a menu and all such things
@@ -29,7 +29,11 @@
 
 typedef struct {
     char    *name;
-    int     (*callback_func)(menu_t *menu,app_t *app, int sel);
+    int    (*callback_func)(menu_t *menu,app_t *app, int sel);
+    int      num_options;
+    char   **options;
+    int      type;
+    void    *value_ptr;
 } option_t;
 
 
@@ -120,6 +124,22 @@ void menu_set_title(menu_t *menu, char *text)
     menu->title = strdup(text);
 }
 
+
+void menu_add_option(menu_t *menu, char *name, int (callback_func)(menu_t *menu,app_t *app, int sel),
+                     int type, int num_options, char **options, void *value_ptr)
+{
+    int     i = menu->numoptions++;
+
+    menu->options = realloc(menu->options,menu->numoptions * sizeof(option_t));
+
+    menu->options[i].name = strdup(name);
+    menu->options[i].callback_func = callback_func;
+    menu->options[i].type = type;
+    menu->options[i].num_options = num_options;
+    menu->options[i].options = options;
+    menu->options[i].value_ptr = value_ptr;
+}
+
 void menu_add(menu_t *menu, char *name, int (*callback_func)(menu_t *menu,app_t *app, int sel))
 {
     int     i = menu->numoptions++;
@@ -128,6 +148,9 @@ void menu_add(menu_t *menu, char *name, int (*callback_func)(menu_t *menu,app_t
 
     menu->options[i].name = strdup(name);
     menu->options[i].callback_func = callback_func;
+    menu->options[i].type          = MENU_NONE;
+    menu->options[i].num_options = 0;
+    menu->options[i].value_ptr = NULL;
 }
 
 
@@ -136,6 +159,7 @@ void menu_add(menu_t *menu, char *name, int (*callback_func)(menu_t *menu,app_t
 void menu_display(menu_t *menu)
 {
     char          buf[256];
+    char         *extra;
     render_t     *render = menu->render;
     int           row;
     int           i;
@@ -145,6 +169,7 @@ void menu_display(menu_t *menu)
         menu = menu->child;
     }
 
+    printf("Displaying menu\n");
     render_set_bgcol(render,BLACK);
     render_clear(render);
 
@@ -157,11 +182,17 @@ void menu_display(menu_t *menu)
     /* Calculate how many rows to print */
     last = (menu->top_offset + (menu->maxrows - 2) < menu->numoptions) ? menu->top_offset + (menu->maxrows - 2) : menu->numoptions;
     for ( row = 2, i = menu->top_offset; i < last; i++, row++ ) {
-        if ( i < 9 && menu->flags & MENU_NUMBERS ) {
-            snprintf(buf,sizeof(buf)," %d. %s\n",i+1,menu->options[i].name);
+        if ( menu->options[i].type == MENU_INT ) {
+	    printf("option %d val %d\n",i,*((int *)menu->options[i].value_ptr));
+            extra = menu->options[i].options[*((int *)menu->options[i].value_ptr)];
         } else {
-            snprintf(buf,sizeof(buf),"   %s\n",menu->options[i].name);
+            extra = "";
         }
+        if ( i < 9 && menu->flags & MENU_NUMBERS ) {
+            snprintf(buf,sizeof(buf)," %d. %s %s\n",i+1,menu->options[i].name,extra);
+        } else {
+            snprintf(buf,sizeof(buf),"   %s %s\n",menu->options[i].name,extra);
+        }      
 
         if ( i == menu->current_selection ) {
             render_set_bgcol(render,CYAN);
@@ -172,17 +203,20 @@ void menu_display(menu_t *menu)
         }
         render_printrow(render,row,buf);
     }
-    gui_register_keys(menu->gui,KEYS_MENU,menu_keys,menu);
+    gui_register_keys(menu->gui,REGISTER_MENU,menu_keys,menu);
 }
 
 
 static int menu_keys(void *ptr, int code)
 {
-    menu_t   *menu = ptr;
-    int       pos;
-    int     (*func)(menu_t *,app_t *app, int sel);
+    menu_t    *menu = ptr;
+    int        pos;
+    int      (*func)(menu_t *,app_t *app, int sel);
+    option_t  *option;
+    unsigned int       *num;
 
 
+    printf("Got menu key %d\n",code);
     switch ( code ) {
     case kChanUp:
         if ( menu->current_selection > 0 ) {
@@ -192,14 +226,13 @@ static int menu_keys(void *ptr, int code)
             }
         } else {
             menu->current_selection = menu->numoptions - 1;	  
-	    if ( menu->numoptions > menu->maxrows - 2 ) {
-		menu->top_offset = menu->numoptions - ( menu->maxrows - 2);
-	    }
+            if ( menu->numoptions > menu->maxrows - 2 ) {
+                menu->top_offset = menu->numoptions - ( menu->maxrows - 2);
+            }
         }
         menu_display(menu);
         break;
     case kChanDn:
-	printf("%d\n",menu->maxrows - 2);
         if ( menu->current_selection < menu->numoptions - 1 ) {
             menu->current_selection++;
             if ( menu->top_offset + (menu->maxrows - 2) <= menu->current_selection ) {
@@ -211,6 +244,40 @@ static int menu_keys(void *ptr, int code)
         }
         menu_display(menu);
         break;
+    case kVolDn:
+        option = &menu->options[menu->current_selection];
+        if ( option->type == MENU_INT ) {
+	    menu_t *parent = menu->parent;
+            num = option->value_ptr;           
+            *num = ( *num + 1 ) % option->num_options;
+            func = menu->options[menu->current_selection].callback_func;
+            if ( func && func(menu,menu->app,-1) == 0 ) {
+                delete_menu(menu);
+                parent->child = NULL;	
+            }    
+	    menu_display(parent);
+  
+        }
+        break;
+    case kVolUp:
+        option = &menu->options[menu->current_selection];
+        if ( option->type == MENU_INT ) {
+	    menu_t *parent = menu->parent;
+            num = option->value_ptr;
+            if ( *num == 0 ) {
+                *num = option->num_options - 1;
+            } else {
+                *num = ( *num - 1 ) % option->num_options;
+            }
+            func = menu->options[menu->current_selection].callback_func;
+            if ( func && func(menu,menu->app,-1) == 0 ) {
+                delete_menu(menu);
+                parent->child = NULL;	
+            }      
+	    menu_display(parent);
+
+        }
+        break;
     case kBack:
         if ( menu->parent ) {
             menu_t *parent = menu->parent;
@@ -224,19 +291,19 @@ static int menu_keys(void *ptr, int code)
             menu_t *parent = menu->parent;
             delete_menu(menu);
             menu = parent;
-	    menu->child = NULL;
+            menu->child = NULL;
         }
         menu_display(menu);
         break;
     case kOk:
         if ( menu->options != NULL ) {
-	    menu_t  *parent;
+            menu_t  *parent;
             func = menu->options[menu->current_selection].callback_func;
             if ( func && func(menu,menu->app,menu->current_selection) == 0 ) {
-		parent = menu->parent;
+                parent = menu->parent;
                 delete_menu(menu);
-		parent->child = NULL;	
-		menu_display(parent);
+                parent->child = NULL;	
+                menu_display(parent);
             }        
         }
         break;
@@ -256,6 +323,8 @@ static int menu_keys(void *ptr, int code)
     }
     return 0;
 }
+
+
  
 /*
  * Local Variables:
