@@ -3,8 +3,8 @@
  *
  *   (C) 2003 Dominic Morris
  *
- *   $Id: stream.c,v 1.14 2003/11/19 16:25:54 dom Exp $
- *   $Date: 2003/11/19 16:25:54 $
+ *   $Id: stream.c,v 1.17 2003/11/24 20:18:40 dom Exp $
+ *   $Date: 2003/11/24 20:18:40 $
  *
  *
  *   Handles the media stream socket..all a bit of mess at the moment..
@@ -23,6 +23,7 @@ struct _stream {
     uint32_t       blocklen;
     int            mediaid;
     int            mediatype;
+    int            last_mediatype;
     char           inbuf[2048];
     int            inbufstart;
     int            mediafd;
@@ -300,7 +301,6 @@ static int media_stop(stream_t *stream, unsigned char *buf, int len)
         return 0;
     }
     printf("Stop\n");
-    dump_hex(buf,len);
 
     if ( stream->mediafd ) {
         close(stream->mediafd);
@@ -319,7 +319,8 @@ static int media_stop(stream_t *stream, unsigned char *buf, int len)
         }
     }
 
-    gui_notify_stop(stream->dongle->gui);
+    stream->last_mediatype = stream->mediatype;
+    stream->mediatype = 0;
 
     ptr = buf + 18;   
     INT16_TO_BUF(stream->mediaid,ptr);
@@ -582,7 +583,7 @@ static void stream_write_data(stream_t *stream, bool_t flush)
 
 /** \brief Return the streamtype
  *
- *  \param gui Handle to the gui channel
+ *  \param stream Handle to the stream channel
  *
  *  \return The current streamtype
  */
@@ -591,6 +592,17 @@ int stream_get_type(stream_t *stream)
     return ( stream->mediatype );
 }
 
+/** \brief Return the last streamtype
+ *
+ *  \param stream Handle to the stream channel
+ *
+ *  \return The current streamtype
+ */
+int stream_get_lasttype(stream_t *stream)
+{
+    return ( stream->last_mediatype );
+}
+
 /*
  * Local Variables:
  *  indent-tabs-mode:nil
