@@ -3,8 +3,8 @@
  *
  *   (C) 2003 Dominic Morris
  *
- *   $Id: udp.c,v 1.10 2003/11/19 16:25:54 dom Exp $
- *   $Date: 2003/11/19 16:25:54 $
+ *   $Id: udp.c,v 1.11 2003/11/24 00:30:27 dom Exp $
+ *   $Date: 2003/11/24 00:30:27 $
  *
  *
  *   Handles the initial UDP phase
@@ -136,7 +136,7 @@ static void udp_read(int fd, short event, void *arg)
 
         /* FIXME Here to support multiple clients */
         prot.guiserv_addr = ntohl(c_gui_hostip);
-        prot.guiserv_port = c_gui_port;
+        prot.guiserv_port = c_gui_port - 100;
         prot.conserv_addr = ntohl(c_stream_hostip);
         prot.conserv_port = c_stream_port;
         prot.serv_addr = ntohl(main_interfaceip);  /* No idea what this is.. */
